#ifdef __KERNEL__
    #include <linux/module.h>
    #include <linux/ioctl.h>    
#else
    #include <stdio.h>
    #include <sys/ioctl.h>  
#endif

#include "cmd_handler.h"

#include "cp15.h"
#include "vfpu.h"
#include "psr.h"
#include "aarch64.h"

func_table_entry func_table[] =
{   
    /* ID Handlers */
    { MS_SYSREG_MIDR, do_MIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CTR, do_CTR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_TCMTR, do_TCMTR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_TLBTR, do_TLBTR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MPIDR, do_MPIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_REVIDR, do_REVIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_AFR0, do_AFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MMFR0, do_MMFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_MMFR1, do_MMFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MMFR2, do_MMFR2_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MMFR3, do_MMFR3_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_PFR0, do_PFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PFR1, do_PFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ISAR0, do_ISAR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ISAR1, do_ISAR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ISAR2, do_ISAR2_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ISAR3, do_ISAR3_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ISAR4, do_ISAR4_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ISAR5, do_ISAR5_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CLIDR, do_CLIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_AIDR, do_AIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CSSELR, do_CSSELR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DFR0, do_DFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CCSIDR, do_CCSIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 }, 
    
    /* Virtual Memory Handlers */
    { MS_SYSREG_SCTLR, do_SCTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PRRR, do_PRRR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MAIR, do_MAIR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_MAIR0, do_MAIR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MAIR1, do_MAIR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_NMRR, do_NMRR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_TTBCR, do_TTBCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },    
    { MS_SYSREG_TTBR0, do_TTBR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_TTBR0LPAE, do_TTBR0LPAE_read, FUNCTYPE_OUT_2, FUNCTYPE_BITS_32 },
    { MS_SYSREG_TTBR1, do_TTBR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_TTBR1LPAE, do_TTBR1LPAE_read, FUNCTYPE_OUT_2, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CONTEXTIDR, do_CONTEXTIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_AMAIR0, do_AMAIR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_AMAIR1, do_AMAIR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_AMAIR, do_AMAIR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_DACR, do_DACR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DCZID, do_DCZID_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    
    /* Fault Handling Handlers */
    { MS_SYSREG_IFAR, do_IFAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_IFSR, do_IFSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ADFSR, do_ADFSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_AIFSR, do_AIFSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DFAR, do_DFAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DFSR, do_DFSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 }, 
    { MS_SYSREG_FAR, do_FAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },   
    
    /* Performance Monitor Handlers */
    { MS_SYSREG_PMCCFILTR, do_PMCCFILTR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMCR, do_PMCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMCCNTR, do_PMCCNTR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_PMCEID0, do_PMCEID0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMCEID1, do_PMCEID1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMCNTENCLR, do_PMCNTENCLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMCNTENSET, do_PMCNTENSET_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 }, 
    { MS_SYSREG_PMINTENCLR, do_PMINTENCLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMINTENSET, do_PMINTENSET_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMOVSR, do_PMOVSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMOVSCLR, do_PMOVSCLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMOVSSET, do_PMOVSSET_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMSELR, do_PMSELR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMSWINC, do_PMSWINC_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMUSERENR, do_PMUSERENR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMXEVCNTR, do_PMXEVCNTR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_PMXEVTYPER, do_PMXEVTYPER_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    
    /* Security Extension Handlers */       
    { MS_SYSREG_VBAR, do_VBAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_ISR, do_ISR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_MVBAR, do_MVBAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_NSACR, do_NSACR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_SCR, do_SCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_SDER, do_SDER_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 }, 
    
    /* "Other" Handlers */  
    { MS_SYSREG_CPACR, do_CPACR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_FCSEIDR, do_FCSEIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_RMR, do_RMR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_RVBAR, do_RVBAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    
    /* Address Translation Handlers */  
    { MS_SYSREG_PAR, do_PAR_read, FUNCTYPE_OUT_1 , FUNCTYPE_BITS_ARCH },
    
    /* Floating Point Handlers */
    { MS_SYSREG_FPEXC, do_FPEXC_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },       
    { MS_SYSREG_FPSCR, do_FPSCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_FPSID, do_FPSID_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_MVFR0, do_MVFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_MVFR1, do_MVFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_MVFR2, do_MVFR2_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
        
    /* Debug Handlers */
    { MS_SYSREG_DBGDIDR, do_DBGDIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DBGAUTHSTATUS, do_DBGAUTHSTATUS_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DBGDRAR, do_DBGDRAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_DBGDSCR, do_DBGDSCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    
    /* Misc */
    { MS_SYSREG_TPIDRPRW, do_TPIDRPRW_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_TPIDRURO, do_TPIDRURO_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_TPIDRURW, do_TPIDRURW_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
    
    /* Jazelle */
    { MS_SYSREG_JIDR, do_JIDR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },     
    { MS_SYSREG_JMCR, do_JMCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_JOSCR, do_JOSCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },       
    
    /* Thumb */
    { MS_SYSREG_TEECR, do_TEECR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_TEEHBR, do_TEEHBR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    
    /* Implementation Defined */
    { MS_SYSREG_CPUMERRSR, do_CPUMERRSR_read, FUNCTYPE_OUT_2, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_L2MERRSR, do_L2MERRSR_read, FUNCTYPE_OUT_2, FUNCTYPE_BITS_ARCH },
    { MS_SYSREG_L2ACTLR, do_L2ACTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ACTLR, do_ACTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_ACTLR2, do_ACTLR2_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CPUACTLR, do_CPUACTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_L2PFR, do_L2PFR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_L2CTLR, do_L2CTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_L2ECTLR, do_L2ECTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },   
    { MS_SYSREG_CPUECTLR, do_CPUECTLR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_IL1DATA0, do_IL1DATA0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_IL1DATA1, do_IL1DATA1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_IL1DATA2, do_IL1DATA2_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_IL1DATA3, do_IL1DATA3_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DL1DATA0, do_DL1DATA0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DL1DATA1, do_DL1DATA1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DL1DATA2, do_DL1DATA2_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DL1DATA3, do_DL1DATA3_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CBAR, do_CBAR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_ARCH },
        
    /* AARCH64 */   
    { MS_SYSREG_AA64AFR0, do_AA64AFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64AFR1, do_AA64AFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64DFR0, do_AA64DFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64DFR1, do_AA64DFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64ISAR0, do_AA64ISAR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64ISAR1, do_AA64ISAR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64MMFR0, do_AA64MMFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64MMFR1, do_AA64MMFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64PFR0, do_AA64PFR0_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    { MS_SYSREG_AA64PFR1, do_AA64PFR1_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_64 },
    
    /* Special purpose registers */
    { MS_SYSREG_CURRENTEL, do_CURRENTEL_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_DAIF, do_DAIF_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_FPCR, do_FPCR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_FPSR, do_FPSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_NZCV, do_NZCV_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_SPSEL, do_SPSEL_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    
    /* Program Status Register */
    { MS_SYSREG_APSR, do_APSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_CPSR, do_CPSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    { MS_SYSREG_SPSR, do_SPSR_read, FUNCTYPE_OUT_1, FUNCTYPE_BITS_32 },
    
    { 0, NULL, 0 , 0},  
};

int
return_table_size(void)
{
    return sizeof(func_table) / sizeof(func_table_entry);
}
