#include <stdio.h>
#include <sys/ioctl.h>

#include "maplesyrup.h"
#include "parse.h"
#include "bitfield_cortex-a15.h"

bitfield_info bitfield_cortex_a15_table[] =
{       
    { 1, 0, MS_SYSREG_REVIDR, NS_PRIVILEGE_LEVEL_1, 0, 32, "ID Number", "ident", "REVIDR", "Revision information" },
    
    { 1, 0, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 0, 2, "CPU ID", "ident", "MPIDR", "Processor number" },
    { 1, 1, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 2, 6, "Reserved", "ident", "MPIDR", "Reserved" },
    { 1, 2, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 8, 4, "Cluster ID", "ident", "MPIDR", "Cluster ID" },
    { 1, 3, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 12, 12, "Reserved", "ident", "MPIDR", "Reserved" },
    { 1, 4, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 24, 1, "MT", "ident", "MPIDR", "Multi-threaded" },
    { 1, 5, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 25, 5, "Reserved", "ident", "MPIDR", "Reserved" },
    { 1, 6, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 30, 1, "U", "ident", "MPIDR", "Uniprocessor system" },
    { 1, 7, MS_SYSREG_MPIDR, NS_PRIVILEGE_LEVEL_1, 31, 1, "ME", "ident", "MPIDR", "Multiprocessor Extensions" }, 
    
    { 1, 0, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 0, 1, "BTB", "other", "ACTLR", "Enables invalidate of BTB" },
    { 1, 1, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 1, 1, "DLB", "other", "ACTLR", "Disable loop buffer" },
    { 1, 2, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 2, 1, "DLBF", "other", "ACTLR", "Limit to one loop buffer detect per flush" },
    { 1, 3, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 3, 1, "DMBTB", "other", "ACTLR", "Disables micro-Branch Target Buffer" },
    { 1, 4, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 4, 1, "DIP", "other", "ACTLR", "Disable interrupt predictor" },
    { 1, 5, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 5, 1, "EPLDNOP", "other", "ACTLR", "Execute PLD instruction as a NOP" },
    { 1, 6, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 6, 1, "SMP", "other", "ACTLR", "Enables SMP" },
    { 1, 7, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 7, 1, "EWFENOP", "other", "ACTLR", "Executes WFE instruction as a NOP instruction" },
    { 1, 8, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 8, 1, "EWFINOP", "other", "ACTLR", "Executes WFI instruction as a NOP instruction" },
    { 1, 9, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 9, 1, "DFRO", "other", "ACTLR", "Disable flag renaming optimization" },
    { 1, 10, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 10, 1, "FS", "other", "ACTLR", "Force serialization after each instruction group" },
    { 1, 11, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 11, 1, "L1PIG", "other", "ACTLR", "Limit to one instruction per instruction group" },
    { 1, 12, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 12, 1, "FPCP", "other", "ACTLR", "Force push of CP14 and CP15 registers" },
    { 1, 13, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 13, 1, "FLCP", "other", "ACTLR", "Flush after CP14 and CP15 writes" },
    { 1, 14, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 14, 1, "FL", "other", "ACTLR", "Force limit of one instruction group commit/deallocate per cycle" },
    { 1, 15, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 15, 1, "FIOBEU", "other", "ACTLR", "Force in-order issue in branch execution unit" },
    { 1, 16, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 16, 1, "EFSO", "other", "ACTLR", "Enable full strongly-ordered and device load replay" },
    { 1, 17, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 17, 1, "DL2_01", "other", "ACTLR", "Disables L2 TLB performance optimizations" }, 
    { 1, 18, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 18, 1, "DL2_02", "other", "ACTLR", "Disables L2 stage 1 translation table walk L2 PA cache" },
    { 1, 19, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 19, 1, "DL2_03", "other", "ACTLR", "Disable L2 stage 1 translation table walk cache" },
    { 1, 20, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 20, 1, "DL2_04", "other", "ACTLR", "Disable L2 translation table walk IPA PA cache" },
    { 1, 21, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 21, 1, "DL2_05", "other", "ACTLR", "Disable L2 TLB prefetching" },
    { 1, 22, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 22, 1, "FIOL", "other", "ACTLR", "Force in-order load issue" }, 
    { 1, 23, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 23, 1, "FIOR", "other", "ACTLR", "Force in-order requests to the same set and way" },
    { 1, 24, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 24, 1, "NCSE", "other", "ACTLR", "Non-cacheable streaming enhancement" },
    { 1, 25, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 25, 2, "WSL1T", "other", "ACTLR", "Write streaming no L1-allocate threshold" },
    { 1, 26, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 27, 2, "WSNAT", "other", "ACTLR", "Write streaming no allocate threshold" },
    { 1, 27, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 29, 1, "FNCEA", "other", "ACTLR", "Force NEON/VFP clock enable active" },
    { 1, 28, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 30, 1, "FMCEA", "other", "ACTLR", "Force main clock enable active" },
    { 1, 29, MS_SYSREG_ACTLR, NS_PRIVILEGE_LEVEL_1, 31, 1, "SDEH", "other", "ACTLR", "Snoop-delayed exclusive handling" },  
    
    { 1, 0, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 0, 10, "Reserved", "secext", "NSACR", "Reserved" },
    { 1, 1, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 10, 1, "CP10", "secext", "NSACR", "Non-secure access to coprocessor 10 enable" },
    { 1, 2, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 11, 1, "CP11", "secext", "NSACR", "Non-secure access to coprecessor 11 enable" },
    { 1, 3, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 12, 3, "Reserved", "secext", "NSACR", "Reserved" },
    { 1, 4, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 15, 1, "NSASEDIS", "secext", "NSACR", "Disable non-secure Advanced SIMD functionality" },
    { 1, 5, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 16, 1, "Reserved", "secext", "NSACR", "Reserved" },
    { 1, 6, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 17, 1, "NS_L2ERR", "secext", "NSACR", "Allow write to L2 AXI Async Error bit in L2ECTLR in non-secure state" },
    { 1, 7, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 18, 1, "NS_SMP", "secext", "NSACR", "Allow write to SMP bit in ACTLR in non-secure state" },
    { 1, 8, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 19, 1, "Reserved", "secext", "NSACR", "Reserved" },
    { 1, 9, MS_SYSREG_NSACR, NS_PRIVILEGE_LEVEL_1, 20, 12, "Reserved", "secext", "NSACR", "Reserved" },
    
    { 1, 0, MS_SYSREG_ADFSR, NS_PRIVILEGE_LEVEL_1, 0, 18, "Index", "fault", "ADFSR", "Index in RAM of the L1 ECC double-bit error" },
    { 1, 1, MS_SYSREG_ADFSR, NS_PRIVILEGE_LEVEL_1, 18, 5, "BankWay", "fault", "ADFSR", "Bank or way of RAM the L1 ECC double-bit error occurred" },
    { 1, 2, MS_SYSREG_ADFSR, NS_PRIVILEGE_LEVEL_1, 23, 1, "L2Error", "fault", "ADFSR", "Indicates an L2 ECC double-bit error occurred" },
    { 1, 3, MS_SYSREG_ADFSR, NS_PRIVILEGE_LEVEL_1, 24, 7, "RAMID", "fault", "ADFSR", "Indicates which RAM the L1 ECC double-bit error occurred in" },
    { 1, 4, MS_SYSREG_ADFSR, NS_PRIVILEGE_LEVEL_1, 31, 1, "Valid", "fault", "ADFSR", "Indicates that an L1 or L2 double-bit error has occurred" },    
    
    { 1, 0, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 0, 3, "Data RAM latency", "impl_def", "L2CTLR", "L2 data RAM latency" },
    { 1, 1, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 3, 2, "Reserved", "impl_def", "L2CTLR", "Reserved" },
    { 1, 2, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 5, 1, "Data RAM setup", "impl_def", "L2CTLR", "L2 data RAM setup" },
    { 1, 3, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 6, 3, "Tag RAM latency", "impl_def", "L2CTLR", "Tag RAM latency" },
    { 1, 4, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 9, 1, "Tag RAM setup", "impl_def", "L2CTLR", "L2 tag RAM setup" },
    { 1, 5, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 10, 2, "Data RAM slice", "impl_def", "L2CTLR", "L2 data RAM slice" },
    { 1, 6, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 12, 1, "Tag RAM slice", "impl_def", "L2CTLR", "L2 tag RAM slice" },
    { 1, 7, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 13, 8, "Reserved", "impl_def", "L2CTLR", "Reserved" },
    { 1, 8, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 21, 1, "ECC and parity enable", "impl_def", "L2CTLR", "ECC and parity enable in L1 and L2 caches" },
    { 1, 9, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 22, 1, "Reserved", "impl_def", "L2CTLR", "Reserved" },
    { 1, 10, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 23, 1, "Interrupt Controller", "impl_def", "L2CTLR", "Interrupt Controller present" },
    { 1, 11, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 24, 2, "Number of processors", "impl_def", "L2CTLR", "Number of processors present" },
    { 1, 12, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 26, 5, "Reserved", "impl_def", "L2CTLR", "Reserved" },
    { 1, 13, MS_SYSREG_L2CTLR, NS_PRIVILEGE_LEVEL_1, 31, 1, "L2RSTDISABLE", "impl_def", "L2CTLR", "Monitors the L2 hardware reset disable pin" },  
    
    
    { 1, 0, MS_SYSREG_IL1DATA0, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "IL1DATA0", "Holds the instruction side L1 array information" },   
    { 1, 1, MS_SYSREG_IL1DATA1, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "IL1DATA1", "Holds the instruction side L1 array information" },   
    { 1, 2, MS_SYSREG_IL1DATA2, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "IL1DATA2", "Holds the instruction side L1 array information" },   
    { 1, 3, MS_SYSREG_IL1DATA3, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "IL1DATA3", "Holds the instruction side L1 array information" },   
    
    { 1, 0, MS_SYSREG_DL1DATA0, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "DL1DATA0", "Holds the data side L1 or L2 array information" },    
    { 1, 1, MS_SYSREG_DL1DATA1, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "DL1DATA1", "Holds the data side L1 or L2 array information" },    
    { 1, 2, MS_SYSREG_DL1DATA2, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "DL1DATA2", "Holds the data side L1 or L2 array information" },    
    { 1, 3, MS_SYSREG_DL1DATA3, NS_PRIVILEGE_LEVEL_1, 0, 32, "Data", "impl_def", "DL1DATA3", "Holds the data side L1 or L2 array information" },    
    
    { 1, 0, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 0, 1, "DPF", "impl_def", "L2ACTLR", "Disable prefetch fowarding" },  
    { 1, 1, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 1, 1, "ARTT", "impl_def", "L2ACTLR", "Enable arbitration replay threshold timeout" },   
    { 1, 2, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 2, 1, "LTB", "impl_def", "L2ACTLR", "Limit to one request per tag bank" },   
    { 1, 3, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 3, 1, "DCEPTE", "impl_def", "L2ACTLR", "Disable clean/evict push to external" },  
    { 1, 4, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 4, 1, "DWU", "impl_def", "L2ACTLR", "Disable WriteUnique and WriteLineUnique transactions from master" },    
    { 1, 5, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 5, 1, "Reserved", "impl_def", "L2ACTLR", "Reserved" },   
    { 1, 6, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 6, 1, "DST", "impl_def", "L2ACTLR", "Disable shareable transactions from master" },  
    { 1, 7, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 7, 1, "EHDT", "impl_def", "L2ACTLR", "Enable hazard detect timeout" },  
    { 1, 8, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 8, 1, "DDVM", "impl_def", "L2ACTLR", "Disable DVM/CMO message broadcast" }, 
    { 1, 9, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 9, 1, "Reserved", "impl_def", "L2ACTLR", "Reserved" },   
    { 1, 10, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 10, 1, "DNSDAR", "impl_def", "L2ACTLR", "Disable Non-secure debug array read" },  
    { 1, 11, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 11, 1, "DDSB", "impl_def", "L2ACTLR", "Disable DSB with no DVM synchronization" },  
    { 1, 12, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 12, 1, "DMOWC", "impl_def", "L2ACTLR", "Disable multiple outstanding WriteClean/WriteBack/Evicts using the same AWID" },   
    { 1, 13, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 13, 1, "DSCDT", "impl_def", "L2ACTLR", "Disable SharedClean data transfers" }, 
    { 1, 14, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 14, 1, "EUCEWD", "impl_def", "L2ACTLR", "Enable UniqueClean evictions with data" },   
    { 1, 15, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 15, 1, "ECPUWFI", "impl_def", "L2ACTLR", "Enable CPU WFI retention mode" },  
    { 1, 16, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 16, 1, "ERTSICT", "impl_def", "L2ACTLR", "Enable replay threshold single issue - current tag bank" },    
    { 1, 17, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 17, 8, "Reserved", "impl_def", "L2ACTLR", "Reserved" },  
    { 1, 18, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 25, 1, "ERTSIAT", "impl_def", "L2ACTLR", "Enable replay threshold single issue - all tag banks" },   
    { 1, 19, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 26, 1, "ERCG", "impl_def", "L2ACTLR", "Enable L2, GIC, and Timer regional clock gates" },   
    { 1, 20, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 27, 1, "FL2CEA", "impl_def", "L2ACTLR", "Force L2 logic clock enable active" },   
    { 1, 21, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 28, 1, "FL2TBCLE", "impl_def", "L2ACTLR", "Force L2 tag bank clock enable active" },    
    { 1, 22, MS_SYSREG_L2ACTLR, NS_PRIVILEGE_LEVEL_1, 29, 3, "Reserved", "impl_def", "L2ACTLR", "Reserved" },      
    
    { 1, 0, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 0, 4, "Reserved", "impl_def", "L2PFR", "Reserved" },     
    { 1, 1, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 4, 2, "L2LSDPD", "impl_def", "L2PFR", "L2 load/store data prefetch distance" },      
    { 1, 2, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 6, 1, "Reserved", "impl_def", "L2PFR", "Reserved" },     
    { 1, 3, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 7, 2, "L2IFPD", "impl_def", "L2PFR", "L2 instruction fetch prefetch distance" },      
    { 1, 4, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 9, 1, "Reserved", "impl_def", "L2PFR", "Reserved" },     
    { 1, 5, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 10, 1, "DTWDAP", "impl_def", "L2PFR", "Disable table walk descriptor across prefetch" },      
    { 1, 6, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 11, 1, "EPRFRT", "impl_def", "L2PFR", "Enable prefetch requests from ReadUnique transactions" },      
    { 1, 7, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 12, 1, "DDTOLSPR", "impl_def", "L2PFR", "Disable dynamic throttling of load/store prefetch requests" },     
    { 1, 8, MS_SYSREG_L2PFR, NS_PRIVILEGE_LEVEL_1, 13, 19, "Reserved", "impl_def", "L2PFR", "Reserved" },       
    
    { 1, 0, MS_SYSREG_ACTLR2, NS_PRIVILEGE_LEVEL_1, 0, 1, "EDCCADC", "impl_def", "ACTLR2", "Execute data cache clean as data cache clean/invalidate" },        
    { 1, 1, MS_SYSREG_ACTLR2, NS_PRIVILEGE_LEVEL_1, 1, 30, "Reserved", "impl_def", "ACTLR2", "Reserved" },        
    { 1, 2, MS_SYSREG_ACTLR2, NS_PRIVILEGE_LEVEL_1, 31, 1, "ECPURCG", "impl_def", "ACTLR2", "Enable CPU regional clock gates" },           
    
    { 1, 0, MS_SYSREG_CBAR, NS_PRIVILEGE_LEVEL_1, 0, 8, "PERIPHBASE_39_32", "impl_def", "CBAR", "Determines reset value" },
    { 1, 1, MS_SYSREG_CBAR, NS_PRIVILEGE_LEVEL_1, 8, 7, "Reserved", "impl_def", "CBAR", "Reserved" },
    { 1, 2, MS_SYSREG_CBAR, NS_PRIVILEGE_LEVEL_1, 15, 16, "PERIPHBASE_31_15", "impl_def", "CBAR", "Determines reset value" },
        
    { 1, 0, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 0, 18, "Index", "impl_def", "CPUMERRSR", "Index address of first memory error" },
    { 1, 1, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 18, 5, "BankWay", "impl_def", "CPUMERRSR", "Bank or way of the RAM where the first memory error occurred" },
    { 1, 2, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 23, 1, "Reserved", "impl_def", "CPUMERRSR", "Reserved" },
    { 1, 3, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 24, 7, "RAMID", "impl_def", "CPUMERRSR", "Indicates the RAM of the first memory error" },
    { 1, 4, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 31, 1, "Valid", "impl_def", "CPUMERRSR", "Set on first memory error" },
    { 1, 5, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 32, 38, "REPEATERROR", "impl_def", "CPUMERRSR", "Repeat error count" },
    { 1, 6, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 40, 8, "OTHERERROR", "impl_def", "CPUMERRSR", "Other error count" },
    { 1, 7, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 48, 14, "Reserved", "impl_def", "CPUMERRSR", "Reserved" },
    { 1, 8, MS_SYSREG_CPUMERRSR, NS_PRIVILEGE_LEVEL_1, 63, 1, "Fatal", "impl_def", "CPUMERRSR", "Fatal bit" },
    
    { 1, 0, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 0, 18, "Index", "impl_def", "L2MERRSR", "Index address of first memory error" },
    { 1, 1, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 18, 5, "CPUIDWay", "impl_def", "L2MERRSR", "Processor and way of the RAM where the first memory error occurred" },
    { 1, 2, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 23, 1, "Reserved", "impl_def", "L2MERRSR", "Reserved" },
    { 1, 3, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 24, 7, "RAMID", "impl_def", "L2MERRSR", "Indicates the RAM of the first memory error" },
    { 1, 4, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 31, 1, "Valid", "impl_def", "L2MERRSR", "Set on first memory error" },
    { 1, 5, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 32, 38, "REPEATERROR", "impl_def", "L2MERRSR", "Repeat error count" },
    { 1, 6, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 40, 8, "OTHERERROR", "impl_def", "L2MERRSR", "Other error count" },
    { 1, 7, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 48, 14, "Reserved", "impl_def", "L2MERRSR", "Reserved" },
    { 1, 8, MS_SYSREG_L2MERRSR, NS_PRIVILEGE_LEVEL_1, 63, 1, "Fatal", "impl_def", "L2MERRSR", "Fatal bit" },
    
};

int
return_bitfield_cortex_a15_size(void)
{
    return (sizeof(bitfield_cortex_a15_table) / sizeof(bitfield_info));
}
